from flask import Flask, render_template, request, redirect, url_for, flash, make_response, session, abort, send_file
from flask_sqlalchemy import SQLAlchemy
from flask_login import LoginManager, UserMixin, login_user, logout_user, login_required, current_user
from flask_babel import Babel, _
from werkzeug.security import generate_password_hash, check_password_hash
from datetime import datetime
import os
import io
import csv

app = Flask(__name__)
app.config['SECRET_KEY'] = os.urandom(24)
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:////app/data/games.db'
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.config['BABEL_DEFAULT_LOCALE'] = 'de'
app.config['BABEL_SUPPORTED_LOCALES'] = ['de', 'en']
app.config['BABEL_TRANSLATION_DIRECTORIES'] = 'translations'

db = SQLAlchemy(app)
login_manager = LoginManager(app)
login_manager.login_view = 'login'
babel = Babel(app)

@babel.localeselector
def get_locale():
    if 'lang' in session and session['lang'] in app.config['BABEL_SUPPORTED_LOCALES']:
        return session['lang']
    return request.accept_languages.best_match(app.config['BABEL_SUPPORTED_LOCALES'])

@app.context_processor
def inject_template_vars():
    return dict(
        get_locale=get_locale,
        theme='dark' if request.cookies.get('dark_mode') == 'true' else 'light'
    )

class User(UserMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(100), unique=True)
    password = db.Column(db.String(100))
    games = db.relationship('Game', backref='owner', lazy=True)

class Game(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), nullable=False)
    steam_key = db.Column(db.String(100), nullable=False)
    status = db.Column(db.String(50), nullable=False)
    recipient = db.Column(db.String(100))
    notes = db.Column(db.Text)
    url = db.Column(db.String(200))
    created_at = db.Column(db.DateTime, default=datetime.utcnow)
    redeem_date = db.Column(db.DateTime)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'), nullable=False)
    steam_appid = db.Column(db.String(20))

@login_manager.user_loader
def load_user(user_id):
    return db.session.get(User, int(user_id))

@app.route('/')
@login_required
def index():
    search_query = request.args.get('q', '')
    query = db.session.query(Game).filter_by(user_id=current_user.id)
    if search_query:
        query = query.filter(Game.name.ilike(f'%{search_query}%'))
    games = query.order_by(Game.created_at.desc()).all()
    return render_template('index.html',
                         games=games,
                         format_date=lambda dt: dt.strftime('%d.%m.%Y') if dt else '',
                         search_query=search_query)

@app.route('/set-lang/<lang>')
def set_lang(lang):
    if lang in app.config['BABEL_SUPPORTED_LOCALES']:
        session['lang'] = lang
    return redirect(request.referrer or url_for('index'))

@app.route('/set-theme/<theme>')
def set_theme(theme):
    resp = make_response('', 204)
    resp.set_cookie('dark_mode', 'true' if theme == 'dark' else 'false', max_age=60*60*24*365)
    return resp

@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        user = User.query.filter_by(username=username).first()
        if user and check_password_hash(user.password, password):
            login_user(user)
            return redirect(url_for('index'))
        flash(_('Invalid credentials'), 'danger')
    return render_template('login.html')

@app.route('/register', methods=['GET', 'POST'])
def register():
    if request.method == 'POST':
        username = request.form['username']
        password = generate_password_hash(request.form['password'])
        if User.query.filter_by(username=username).first():
            flash(_('Username already exists'), 'danger')
            return redirect(url_for('register'))
        new_user = User(username=username, password=password)
        db.session.add(new_user)
        db.session.commit()
        login_user(new_user)
        return redirect(url_for('index'))
    return render_template('register.html')

@app.route('/logout')
@login_required
def logout():
    logout_user()
    return redirect(url_for('login'))

import re

def extract_steam_appid(url):
    match = re.search(r'store\.steampowered\.com/app/(\d+)', url or '')
    if match:
        return match.group(1)
    return ''

@app.route('/add', methods=['GET', 'POST'])
@login_required
def add_game():
    if request.method == 'POST':
        try:
            url = request.form.get('url', '')
            steam_appid = request.form.get('steam_appid', '').strip()
            if not steam_appid:
                steam_appid = extract_steam_appid(url)
            new_game = Game(
                name=request.form['name'],
                steam_key=request.form['steam_key'],
                status=request.form['status'],
                recipient=request.form.get('recipient', ''),
                notes=request.form.get('notes', ''),
                url=url,
                steam_appid=steam_appid,
                redeem_date=datetime.strptime(request.form['redeem_date'], '%Y-%m-%d') if request.form['redeem_date'] else None,
                user_id=current_user.id
            )
            db.session.add(new_game)
            db.session.commit()
            flash(_('Game added successfully!'), 'success')
            return redirect(url_for('index'))
        except Exception as e:
            db.session.rollback()
            flash(_('Error: ') + str(e), 'danger')
    return render_template('add_game.html')

@app.route('/edit/<int:game_id>', methods=['GET', 'POST'])
@login_required
def edit_game(game_id):
    game = db.session.get(Game, game_id)
    if not game or game.owner != current_user:
        return _("Not allowed!"), 403

    if request.method == 'POST':
        try:
            url = request.form.get('url', '')
            steam_appid = request.form.get('steam_appid', '').strip()
            if not steam_appid:
                steam_appid = extract_steam_appid(url)
            game.name = request.form['name']
            game.steam_key = request.form['steam_key']
            game.status = request.form['status']
            game.recipient = request.form.get('recipient', '')
            game.notes = request.form.get('notes', '')
            game.url = url
            game.steam_appid = steam_appid
            game.redeem_date = datetime.strptime(request.form['redeem_date'], '%Y-%m-%d') if request.form['redeem_date'] else None
            db.session.commit()
            flash(_('Changes saved!'), 'success')
            return redirect(url_for('index'))
        except Exception as e:
            db.session.rollback()
            flash(_('Error: ') + str(e), 'danger')
    return render_template('edit_game.html',
                          game=game,
                          redeem_date=game.redeem_date.strftime('%Y-%m-%d') if game.redeem_date else '')

@app.route('/delete/<int:game_id>', methods=['POST'])
@login_required
def delete_game(game_id):
    game = Game.query.get_or_404(game_id)
    if game.owner != current_user:
        return _("Not allowed!"), 403
    try:
        db.session.delete(game)
        db.session.commit()
        flash(_('Game deleted!'), 'success')
    except Exception as e:
        db.session.rollback()
        flash(_('Error deleting: ') + str(e), 'danger')
    return redirect(url_for('index'))

# --- Import/Export Funktionen ---

@app.route('/export', methods=['GET'])
@login_required
def export_games():
    games = Game.query.filter_by(user_id=current_user.id).all()
    output = io.StringIO()
    writer = csv.writer(output)
    writer.writerow(['Name', 'Steam Key', 'Status', 'Recipient', 'Notes', 'URL', 'Created', 'Redeem by', 'Steam AppID'])
    for game in games:
        writer.writerow([
            game.name, game.steam_key, game.status, game.recipient, game.notes,
            game.url, game.created_at.strftime('%Y-%m-%d %H:%M:%S') if game.created_at else '',
            game.redeem_date.strftime('%Y-%m-%d') if game.redeem_date else '',
            game.steam_appid
        ])
    output.seek(0)
    return send_file(
        io.BytesIO(output.getvalue().encode('utf-8')),
        mimetype='text/csv',
        as_attachment=True,
        download_name='games_export.csv'
    )

@app.route('/import', methods=['GET', 'POST'])
@login_required
def import_games():
    if request.method == 'POST':
        file = request.files.get('file')
        if file and file.filename.endswith('.csv'):
            stream = io.StringIO(file.stream.read().decode("UTF8"), newline=None)
            reader = csv.DictReader(stream)
            for row in reader:
                new_game = Game(
                    name=row['Name'],
                    steam_key=row['Steam Key'],
                    status=row['Status'],
                    recipient=row.get('Recipient', ''),
                    notes=row.get('Notes', ''),
                    url=row.get('URL', ''),
                    created_at=datetime.strptime(row['Created'], '%Y-%m-%d %H:%M:%S') if row.get('Created') else datetime.utcnow(),
                    redeem_date=datetime.strptime(row['Redeem by'], '%Y-%m-%d') if row.get('Redeem by') else None,
                    steam_appid=row.get('Steam AppID', ''),
                    user_id=current_user.id
                )
                db.session.add(new_game)
            db.session.commit()
            flash(_('Import erfolgreich!'), 'success')
            return redirect(url_for('index'))
        flash(_('Bitte eine gültige CSV-Datei hochladen.'), 'danger')
    return render_template('import.html')

if __name__ == '__main__':
    with app.app_context():
        db.create_all()
    app.run(host='0.0.0.0', port=5000)
